//
//  TRMovieInspector_Selection.m
//  Tryst
//
//  Created by Francisco Tolmasky on Sat Dec 06 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "TRMovieScrollView.h"
#import "TRMovieImageView.h"
#import "TRMovieInspector.h"


static NSRange ClampRangeToRange(NSRange range, NSRange boundary);

@interface TRMovieInspector (EditingFriend)
- (void)setCurrentEditedStartLocation:(unsigned)aLocation;
@end

@implementation TRMovieInspector (Selection)

#pragma mark Selection Management

// validateProposedRange:updateEditingInfo: is a utility routine used by textView:willChangeSelectionFromCharacterRange:toCharacterRange: 
// and selectionRangeForProposedRange:granulatiry:.  'proposedSelectedCharRange' selection range is the range of characters NSTextView 
// would like to make the new selection.  'updateEditingInfo' indicates whether or not this routine should update TRMovieInspector's 
// ivars that indicate editing status.  The return value is the range we want NSTextView to select given the proposedSelectedCharRange.

//  This method takes a proposed selection range and returns the actual range that should be used for selection.  
//  If the user is trying to select an editable range for the first time, we are going to force the selection to be the entire field.  
//  Otherwise, we just try to make sure the selection doesn't fall outside into a non-editable area.

- (NSRange)validateProposedRange:(NSRange)proposedSelectedCharRange updateEditingInfo:(BOOL)updateEditingInfo dragging:(BOOL)isDragging
{
    NSTextStorage *textStorage = [self textStorage];
    NSString *placeholderString= nil;
    NSRange rangeOfPlaceholderString= kNotFoundRange,
            resultRange= NSMakeRange(0,0),
            effectiveRange;
    BOOL isPlaceholder,
         isBeginning;

    // 1. Compute 'rangeOfAttributeForProposal', given 'proposedSelectedCharRange'.

    // We are going to see if the clicked range is covered by the 'editable' attribute.  Given the input range 'proposedSelectedCharRange' 
    // we create a range 'rangeOfAttributeForProposal' that we use to search for the 'editable' attribute.  If we are editing already we 
    // may restrict the search.

    // If we are already editing intersect the input proposed range with the current editing range.
    rangeOfPlaceholderString= proposedSelectedCharRange;
    
    if([self isEditing])
    {
        // The selection length is 0, that means there is no character selection, just a cursor position proposed.  We want to make sure 
        // that, if the cursor is at the end of the current edited field, that our search range is in the edited range.
        [textStorage attribute: kPlaceholderAttributeName atIndex: currentEditedStartLocation effectiveRange: &effectiveRange];
        if (proposedSelectedCharRange.length==0)
        {
            if(proposedSelectedCharRange.location==NSMaxRange(effectiveRange))
                rangeOfPlaceholderString= NSMakeRange(effectiveRange.location, 0);
        }
        else rangeOfPlaceholderString= NSIntersectionRange(rangeOfPlaceholderString, effectiveRange); // handles select all localization withing "pseudo textfield"
    }

    // 2. See if the 'rangeOfAttributeForProposal' is covered by the 'editable' attribute.
    
    // Make sure the range is valid.
    if(NSMaxRange(rangeOfPlaceholderString)<[textStorage length])
    {
        placeholderString= [textStorage attribute: kPlaceholderAttributeName atIndex: rangeOfPlaceholderString.location effectiveRange: &effectiveRange];
        isPlaceholder= [textStorage attribute: NSForegroundColorAttributeName atIndex: effectiveRange.location+1 effectiveRange: NULL]==[NSColor grayColor];
        //if this
        BOOL isEditableRange= (placeholderString && NSEqualRanges(rangeOfPlaceholderString, NSIntersectionRange(rangeOfPlaceholderString, effectiveRange)));
        if(isEditableRange) rangeOfPlaceholderString= effectiveRange;
        else placeholderString= nil;
    }
    
    // 3. If the clicked range is covered by the 'editable' attribute, compute the selection range to return.
    if (placeholderString)
    {
        NSString *currentString= [[textStorage attributedSubstringFromRange: rangeOfPlaceholderString] string];
        if ([placeholderString isEqualToString: currentString] && isPlaceholder || currentEditedStartLocation!=rangeOfPlaceholderString.location && !isDragging)
        {
            // We clicked on a field whose current value is the 'default' editing value (the placeholder string value).
            // ... or we clicked on an 'editable' field which is different from the current editable field.
            // ... So, force the selection to be the entire field.
            resultRange= rangeOfPlaceholderString;
            if(updateEditingInfo)
            {
                // Note: We should probably be more optimal about this.
                [self setNeedsDisplayInRect: [self bounds] avoidAdditionalLayout: NO];
                //currentEditedStartLocation= rangeOfPlaceholderString.location;
                
                if(currentEditedStartLocation!=rangeOfPlaceholderString.location)
                    [self setCurrentEditedStartLocation: rangeOfPlaceholderString.location];
            }
        }
        else
        {
            // We are editing, so we simply need to clamp the selection to the current 'editable' area.
            //resultRange= proposedSelectedCharRange;
            resultRange= ClampRangeToRange(/*resultRange*/proposedSelectedCharRange, rangeOfPlaceholderString);
            if (updateEditingInfo)
            {
                // Note: We should probably be more optimal about this.
                [self setNeedsDisplayInRect: [self bounds] avoidAdditionalLayout: NO];
               
                if(currentEditedStartLocation!=rangeOfPlaceholderString.location)
                    [self setCurrentEditedStartLocation: rangeOfPlaceholderString.location];
            }
        }
    }
    else
    {
        if(currentEditedStartLocation!=NSNotFound)
            [textStorage attribute: kPlaceholderAttributeName atIndex: currentEditedStartLocation effectiveRange: &effectiveRange];
        if (currentEditedStartLocation!=NSNotFound &&
            proposedSelectedCharRange.length==0 &&
           ((isBeginning= proposedSelectedCharRange.location==currentEditedStartLocation-1) ||
            proposedSelectedCharRange.location==NSMaxRange(effectiveRange)+1 ))
            resultRange= NSMakeRange(isBeginning?currentEditedStartLocation:NSMaxRange(effectiveRange),0);
        else
        {
            // We are not editing, and the proposals range didn't include something completely covered by the 'editable' attribute, so don't allow any selection.
            resultRange= NSMakeRange(0,0);
            if (updateEditingInfo)
            {
                // Note: We should probably be more optimal about this.
                [self setNeedsDisplayInRect: [self bounds] avoidAdditionalLayout: NO];
                
                if(currentEditedStartLocation!=NSNotFound) [self setCurrentEditedStartLocation: NSNotFound];
            }
        }
    }

    return resultRange;
}

- (NSRange)validateProposedRange:(NSRange)proposedSelectedCharRange updateEditingInfo:(BOOL)updateEditingInfo
{
    return [self validateProposedRange: proposedSelectedCharRange updateEditingInfo: updateEditingInfo dragging: NO];
}

- (void)textViewDidChangeSelection:(NSNotification *)aNotification
{
    NSTextStorage *textStorage= [self textStorage];
    NSRange effectiveRange;
    
    BOOL isEditingYear=
        [self isEditing]&&[textStorage attribute: kPropertyAttributeName atIndex: currentEditedStartLocation effectiveRange: &effectiveRange]==kTRMovieYearProperty;

    if(isEditingYear!=(currentEditedYearLocation!=NSNotFound))
    {
        if(!isEditingYear)
        {
            effectiveRange= NSMakeRange(currentEditedYearLocation,0);
            [textStorage attribute: kPropertyAttributeName atIndex: effectiveRange.location effectiveRange: &effectiveRange];
            //[textStorage attribute: kIdentifierAttributeName atIndex: NSMaxRange(effectiveRange)+3 effectiveRange: &effectiveRange];
            currentEditedYearLocation= NSNotFound;
        }
        else currentEditedYearLocation= currentEditedStartLocation;
        [textStorage replaceCharactersInRange: NSMakeRange(effectiveRange.location-2,2) withString: isEditingYear?@"( ":@" ("];
        [textStorage replaceCharactersInRange: NSMakeRange(NSMaxRange(effectiveRange),2) withString: isEditingYear?@" )":@") "];
    }
}

- (NSRange)textView:(NSTextView *)textView willChangeSelectionFromCharacterRange:(NSRange)oldSelectedCharRange toCharacterRange:(NSRange)proposedSelectedCharRange
{
    return [self isEditable]?[self validateProposedRange: proposedSelectedCharRange updateEditingInfo: YES]:proposedSelectedCharRange;
}

- (NSRange)selectionRangeForProposedRange:(NSRange)proposedCharRange granularity:(NSSelectionGranularity)granularity
{
    if(![self isEditable]) return proposedCharRange;
    NSRange validProposedCharRange= [self validateProposedRange: proposedCharRange updateEditingInfo: NO];
    return [super selectionRangeForProposedRange: validProposedCharRange granularity: granularity];
}

// --- More Selection Management  ---

// shouldChangeTextInRange:replacementString: makes sure that editable areas are always tagged with the editable attribute.  Further, if 
// the user ever tries to delete all the text in an editable area, the method will replace the text with the placeholder value.
- (BOOL)shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString
{
    NSTextStorage *textStorage= [self textStorage];
    NSRange effectiveRange;
    NSString *placeholderString= [textStorage attribute: kPlaceholderAttributeName atIndex: affectedCharRange.location effectiveRange: &effectiveRange];

    BOOL replacingEntireRange= NSEqualRanges(effectiveRange, affectedCharRange);

    // Make sure paste is in a "pseudo textfield", not just anywhere.
    // Also, don't allow deletions past last position.
    if (currentEditedStartLocation==NSNotFound ||
        effectiveRange.location==affectedCharRange.location && 
        [[textStorage string] characterAtIndex: effectiveRange.location]=='\t' &&
        [replacementString isEqualToString: @""])
    {
        NSBeep();
        return NO; 
    }

    if (replacingEntireRange)
    {
        // We are replacing the entire selected range with a new value.  If the replacement string is @"", then we actually want to insert the 
        // placeholder value into the textStorage.  Otherwise just allow the replacement to happen and make sure it is still tagged with the 
        // editable attribute.
        BOOL deletingAll= [replacementString isEqualToString: @""];
        
        NSString    *property= [textStorage attribute: kPropertyAttributeName atIndex: affectedCharRange.location effectiveRange: NULL],
                    *identifier= [textStorage attribute: kIdentifierAttributeName atIndex: affectedCharRange.location effectiveRange: NULL];
        
        NSFont *font= [textStorage attribute: NSFontAttributeName atIndex: affectedCharRange.location effectiveRange: NULL];
        
        NSString *string= deletingAll?placeholderString:replacementString;
        NSDictionary *attributes= deletingAll?[self placeholderAttributesForProperty: property identifier: identifier placeholderString: placeholderString font: font]:[self typingAttributes];
        NSRange range= affectedCharRange;
        
        // Affix attributes beforehand to avoid change in text without proper color bug.
        NSMutableAttributedString *a_str= [[NSMutableAttributedString alloc] initWithString: string attributes: attributes];
        //[a_str setAttributes: attrs range: NSMakeRange(0,[string length])];//set or add?
        [textStorage replaceCharactersInRange: range withAttributedString: a_str];//this is to avoid change before gray attributes are set
        //[textStorage replaceCharactersInRange: range withString: string];
        //[textStorage setAttributes: attrs range: NSMakeRange(range.location, [string length])];
        if(deletingAll) [self setSelectedRange: range];
        [self didChangeText];
        
        return NO;
    }

    return YES;
}

// --- More Selection Management ---

// In mouseDown: we check to see where the user clicked.  If the clicked in a new editable area, make sure to select the entire range.  Otherwise 
// is we are already editing, just let NSTextView do its normal mouseDown:, which will eventually be handled by 
// validProposedCharRange:updateEditingInfo: above.
- (void)mouseDown:(NSEvent *)theEvent
{
    NSPoint localPoint= [self convertPoint: [theEvent locationInWindow] fromView: nil];
    
    if(![self isEditable])
	{
		[super mouseDown: theEvent];
		return;
    }
	
    NSTextContainer *textContainer= [self textContainer];
    NSTextStorage *textStorage= [self textStorage];
    NSLayoutManager *layoutManager= [textContainer layoutManager];

    unsigned glyphAtPoint = [layoutManager glyphIndexForPoint: localPoint inTextContainer: textContainer];
    unsigned characterAtPoint= [layoutManager characterIndexForGlyphAtIndex: glyphAtPoint];
    
    NSTextAttachment *attachment= [textStorage attribute: NSAttachmentAttributeName atIndex: characterAtPoint effectiveRange: NULL];
    if(attachment)
	{
		[super mouseDown: theEvent];
		return;
    }
	
    NSRange effectiveRange;

    NSString *property= [textStorage attribute: kPropertyAttributeName atIndex: characterAtPoint effectiveRange: &effectiveRange];
    
    if (effectiveRange.location!=currentEditedStartLocation)
    {
        NSRange selectionRange= NSMakeRange(0,0);
        if(property)
        {
            [self setCurrentEditedStartLocation: effectiveRange.location];
            selectionRange= effectiveRange;
        }
        [self setNeedsDisplayInRect: [self bounds] avoidAdditionalLayout: YES];
        [self setSelectedRange: selectionRange];
    }
    else [super mouseDown: theEvent];
}

#pragma mark Editing

- (NSRange)nextEditableRange:(unsigned)currentIndex forward:(BOOL)forward
{
    // A utility that, given an character index, searches forward/backwards to find the next editable range of characters.
    if (currentIndex==NSNotFound) return NSMakeRange(0,0);

    NSTextStorage *textStorage= [self textStorage];
    NSRange effectiveRange;
    [textStorage attribute: kPropertyAttributeName atIndex: currentIndex effectiveRange: &effectiveRange];
    NSRange editedRange= effectiveRange;
    unsigned index= (forward?NSMaxRange(effectiveRange):effectiveRange.location-1);
    NSRange fullRange= NSMakeRange(0, [textStorage length]);

    if ([textStorage attribute: kPropertyAttributeName atIndex: index longestEffectiveRange: &effectiveRange inRange: fullRange])
        return effectiveRange;
    index= (forward?NSMaxRange(effectiveRange):effectiveRange.location-1);
    if (index<[textStorage length] && [textStorage attribute: kPropertyAttributeName atIndex: index longestEffectiveRange: &effectiveRange inRange: fullRange])
        return effectiveRange;

    unsigned loopAroundIndex= (forward?0:NSMaxRange(fullRange)-1);
    if ([textStorage attribute: kPropertyAttributeName atIndex: loopAroundIndex longestEffectiveRange: &effectiveRange inRange: fullRange])
        return effectiveRange;
    index= (forward?NSMaxRange(effectiveRange):effectiveRange.location-1);
    if (index<[textStorage length] && [textStorage attribute: kPropertyAttributeName atIndex: index longestEffectiveRange: &effectiveRange inRange: fullRange])
        return effectiveRange;

    return editedRange;
}

@end

// Ensure range is within "boundary"
static NSRange ClampRangeToRange(NSRange range, NSRange boundary)
{
    NSRange resultRange= range;
    if (range.location<boundary.location)
    {
        unsigned delta= boundary.location-range.location;
        resultRange.length-= delta;
        resultRange.location+= delta;
    }
    if (NSMaxRange(range)>NSMaxRange(boundary))
    {
        unsigned delta= NSMaxRange(range)-NSMaxRange(boundary);
        resultRange.length-= delta;
    }
    return resultRange;
}