/*
 *  NSURL+NDCarbonUtilities.h category
 *  AppleScriptObjectProject
 *
 *  Created by nathan on Wed Dec 05 2001.
 *  Copyright (c) 2001 __CompanyName__. All rights reserved.
 */

#import <Foundation/Foundation.h>
#import <Carbon/Carbon.h>

/*!
	@category NSURL(NDCarbonUtilities)
	@abstract Provides method for interacting with Carbon APIs.
	@discussion Methods for dealing with <TT>FSRef</TT>&rsquo;s, <TT>FSSpec</TT> and other useful carbon stuff.
 */
@interface NSURL (NDCarbonUtilities)

/*!
	@method URLWithFSRef:
	@abstract Alloc and intialize a <TT>NSURL</TT>.
	@discussion Returns a file url for the file refered to by a <TT>FSRef</TT>.
	@param aFsRef A pointer to a <TT>FSRef</TT>.
	@result A <TT>NSURL</TT> containing a file url.
 */
+ (NSURL *)URLWithFSRef:(const FSRef *)aFsRef;

	/*!
@method URLWithFileSystemPathHFSStyle:
	 @abstract Alloc and intialize a <TT>NSURL</TT>.
	 @discussion Returns a file url for the file refered to by a HFS style path.
	 @param aHFSString A <tt>NSString</tt> containing a HFS style path.
	 @result A <TT>NSURL</TT> containing a file url.
	 */
+ (NSURL *)URLWithFileSystemPathHFSStyle:(NSString *)aHFSString;
	/*!
	@method getFSRef:
	@abstract Get a <TT>FSRef</TT>.
	@discussion Obtain a <TT>FSRef</TT> for a file url.
	@param aFsRef A pointer to a <TT>FSRef</TT> struct, to be filled by the method.
	@result Returns <TT>YES</TT> if successful, if the method returns <TT>NO</TT> then <TT>aFsRef</TT> contains garbage.
 */
- (BOOL)getFSRef:(FSRef *)aFsRef;

/*!
	@method getFSSpec:
	@abstract Get a <TT>FSSpec</TT>.
	@discussion Obtain a <TT>FSSpec</TT> for a file url.
	@param aFSSpec A pointer to a <TT>FSSpec</TT> struct, to be filled by the method.
	@result Returns <TT>YES</TT> if successful, if the method returns <TT>NO</TT> then <TT>aFSSpec</TT> contains garbage.
 */
- (BOOL)getFSSpec:(FSSpec *)aFSSpec;

/*!
	@method URLByDeletingLastPathComponent
	@abstract Delete last component of a url.
	@discussion Returns a new <TT>NSURL</TT> equivelent to the receiver with the last component removed.
	@result A new <TT>NSURL</TT>
 */
- (NSURL *)URLByDeletingLastPathComponent;

/*!
	@method fileSystemPathHFSStyle
	@abstract Returns a HFS style path.
	@discussion Returns a <TT>NSString</TT> containg a HFS style path (e.g. <TT>Macitosh HD:Users:</TT>) useful for display purposes.
	@result A new <TT>NSString</TT> containing a HFS style path for the same file or directory as the receiver.
 */
- (NSString *)fileSystemPathHFSStyle;

/*!
	@method resolveAliasFile
	@abstract Resolve an alias file.
	@discussion Returns an file url <TT>NSURL</TT> refered to by the receveive if the receveive refers to an alias file. If it does not refer to an alias file the a url identical to the receveive is returned.
	@result An file url <TT>NSURL</TT>.
 */
- (NSURL *)resolveAliasFile;

/*!
	@method finderInfoFlags:type:creator:
	@abstract Get finder info flags creator and type.
	@discussion The bits of the finder info flag are
	<blockquote>
	<table border = "1"  width = "90%">
		<thead><tr><th>Name</th><th>Description</th></tr></thead>
		<tr><td align = "center"><tt>kIsOnDesk</tt></td><td>Files and folders (System 6)</td><tr>
		<tr><td align = "center"><tt>kColor</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kIsShared</tt></td><td>Files only (Applications only)<br>
					If clear, the application needs to write to its resource fork, and therefore cannot be shared on a server</td><tr>
		<tr><td align = "center"><tt>kHasNoINITs</tt></td><td>Files only (Extensions/Control Panels only)<br>
					This file contains no INIT resource</td><tr>
		<tr><td align = "center"><tt>kHasBeenInited</tt></td><td>Files only<br>
					Clear if the file contains desktop database resources ('BNDL', 'FREF', 'open', 'kind'...) that have not been added yet. Set only by the Finder</td><tr>
		<tr><td align = "center"><tt>kHasCustomIcon</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kIsStationery</tt></td><td>Files only</td><tr>
		<tr><td align = "center"><tt>kNameLocked</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kHasBundle</tt></td><td>Files only</td><tr>
		<tr><td align = "center"><tt>kIsInvisible</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kIsAlias</tt></td><td>Files only.</td><tr>
	</table>
	</blockquote>
	@param aFlags Contains finder flags on return.
	@param aType Contains finder type on return.
	@param aCreator Contains creator on return.
	@result Return <tt>YES</tt> if successful, otherwise <tt>NO</tt> and the returned values are invalid.
 */
- (BOOL)finderInfoFlags:(UInt16*)aFlags type:(OSType*)aType creator:(OSType*)aCreator;

/*!
	@method finderLocation
	@abstract Return a finder items location.
	@discussion Returns a finder items location within its parent window.
	@result A <tt>NSPoint</tt>
 */
- (NSPoint)finderLocation;

/*!
	@method setFinderInfoFlags:mask:type:creator:
	@abstract Set finder info flags, creator and type.
	@discussion The bits of the finder info flag are
	<blockquote>
		<table border = "1"  width = "90%">
		<thead><tr><th>Name</th><th>Description</th></tr></thead>
		<tr><td align = "center"><tt>kIsOnDesk</tt></td><td>Files and folders (System 6)</td><tr>
		<tr><td align = "center"><tt>kColor</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kIsShared</tt></td><td>Files only (Applications only)<br>
		If clear, the application needs to write to its resource fork, and therefore cannot be shared on a server</td><tr>
		<tr><td align = "center"><tt>kHasNoINITs</tt></td><td>Files only (Extensions/Control Panels only)<br>
		This file contains no INIT resource</td><tr>
		<tr><td align = "center"><tt>kHasBeenInited</tt></td><td>Files only<br>
		Clear if the file contains desktop database resources ('BNDL', 'FREF', 'open', 'kind'...) that have not been added yet. Set only by the Finder</td><tr>
		<tr><td align = "center"><tt>kHasCustomIcon</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kIsStationery</tt></td><td>Files only</td><tr>
		<tr><td align = "center"><tt>kNameLocked</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kHasBundle</tt></td><td>Files only</td><tr>
		<tr><td align = "center"><tt>kIsInvisible</tt></td><td>Files and folders</td><tr>
		<tr><td align = "center"><tt>kIsAlias</tt></td><td>Files only.</td><tr>
		</table>
	</blockquote>
	@param aFlags Finder flags.
	@param aMask Mask for Finder flags
	@param aType The Finder file type
	@param aCreator The application creator code
	@result Returns <tt>YES</tt> if successful.
 */
- (BOOL)setFinderInfoFlags:(UInt16)aFlags mask:(UInt16)aMask type:(OSType)aType creator:(OSType)aCreator;

/*!
	@method setFinderLocation:
	@abstract Sets the location a finder item.
	@discussion Set the location of a finder item within in container.
	@param aLocation The location
	@result Returns <tt>YES</tt> if successful.
 */
- (BOOL)setFinderLocation:(NSPoint)aLocation;


@end
